const std = @import("std");

const Bus = @import("../../Bus.zig");
const Arm7tdmi = @import("../../cpu.zig").Arm7tdmi;
const InstrFn = @import("../../cpu.zig").ArmInstrFn;

const sext = @import("../../util.zig").sext;

pub fn branch(comptime L: bool) InstrFn {
    return struct {
        fn inner(cpu: *Arm7tdmi, _: *Bus, opcode: u32) void {
            if (L) cpu.r[14] = cpu.r[15] - 4;

            const offset = sext(u32, u24, opcode) << 2;
            cpu.r[15] = cpu.r[15] +% offset;
            cpu.pipe.flush();
        }
    }.inner;
}

pub fn branchAndExchange(cpu: *Arm7tdmi, _: *Bus, opcode: u32) void {
    const rn = opcode & 0xF;

    if (cpu.r[rn] & 1 == 1) {
        cpu.r[15] = cpu.r[rn] & ~@as(u32, 1);
        cpu.cpsr.t.set();
    } else {
        cpu.r[15] = cpu.r[rn] & ~@as(u32, 3);
        cpu.cpsr.t.unset();
    }

    cpu.pipe.flush();
}
